require 'active_record'
require 'test/unit'
require 'mongoid'
require './lib/mongoid/active_record/eager_loadable.rb'

ENV['MONGOID_ENV'] = 'test'
Mongoid.load!('./config/mongoid.yml', :test)
ActiveRecord::Base.establish_connection(
  adapter: 'sqlite3',
  database: ':memory:'
)

require './test/support/models'
require './test/support/active_record/sql_counter.rb'

ActiveRecord::Base.logger = Logger.new(File.join(File.dirname(__FILE__), "../log/debug.log"))
ActiveSupport::Notifications.subscribe('sql.active_record', ActiveRecord::SQLCounter.new)

class EagerLoadableTest < ActiveRecord::TestCase
  def setup
    @jim      = Person.create!(name: 'Jim')
    @bill     = Person.create!(name: 'Bill')
    @bob      = Person.create!(name: 'Bob')
    @joe      = Person.create!(name: 'Joe')
    @jack     = Person.create!(name: 'Jack')

    @fletch   = Pet.create!(name: 'Fletch', person_id: @jim.id)
    @wilson   = Pet.create!(name: 'Wilson', person_id: @bill.id)
    @mocha    = Pet.create!(name: 'Mocha', person_id: @bob.id)
    @tanner   = Pet.create!(name: 'Tanner', person_id: @joe.id)
    @freckles = Pet.create!(name: 'Freckles', person_id: @jack.id)
  end

  def teardown
    Person.delete_all
    Pet.delete_all
  end

  def test_respond_to_belongs_to_active_record
    assert_respond_to Pet, :belongs_to_active_record
  end

  def test_respond_to_includes_active_record
    assert_respond_to Pet.scoped, :includes_active_record
  end

  def test_mongoid_model_responds_to_association_method
    assert_respond_to Pet.first, :person
  end

  def test_mongoid_model_without_includes_active_record_finds_associated_model_instance
    assert_kind_of Person, Pet.first.person
  end

  def test_mongoid_model_with_includes_active_record_finds_associated_model_instance
    assert_kind_of Person, Pet.scoped.includes_active_record(:person).first.person
  end

  def test_getter_method_returns_proper_associatied_object_without_eager_loading
    assert_equal Pet.first.person, Person.find(Pet.first.person_id)
  end

  def test_getter_method_returns_proper_associatied_object_with_eager_loading
    assert_equal Pet.scoped.includes_active_record(:person).first.person,
                 Person.find(Pet.scoped.includes_active_record(:person).first.person_id)
  end

  def test_multiple_queries_execute_without_eager_loading
    assert_queries(5) do
      Pet.all.map(&:person)
    end
  end

  def test_one_query_executes_with_eager_loading
    assert_queries(1) do
      Pet.scoped.includes_active_record(:person).map(&:person)
    end
  end
end
